#include "inner_storage.h"

#include "jsonobj.h"

namespace json {
    json_t JSON::getType() const {
        return type;
    }

    bool JSON::isNull() const {
        return type == null_symbol;
    }

    bool JSON::isBool() const {
        return type == false_symbol || type == true_symbol;
    }

    bool JSON::isInteger() const {
        return type == integer;
    }

    bool JSON::isFalse() const {
        return type == false_symbol;
    }

    bool JSON::isTrue() const {
         return type == true_symbol;
    }

    bool JSON::isString() const {
        return type == string;
    }

    bool JSON::isArray() const {
        return type == array;
    }

    bool JSON::isDictionary() const {
        return type == dictionary;
    }

    bool JSON::isNatalistic() const {
        return type == array || type == dictionary;
    }

    bool JSON::isSymbol() const {
        return type == null_symbol || type == false_symbol || type == true_symbol;
    }

    bool JSON::toBool() const {
        switch(type) {
            case false_symbol:
                return false;
            case true_symbol:
                return true;
            default:
                throw misuse("json obj is not boolean");
        }
    }

    const Integer& JSON::asInteger() const {
        if (isInteger())
            return *((const Integer*)value);
        throw misuse("json obj is not integer");
    }

    const std::string& JSON::asString() const {
        if (isString())
            return *(const std::string*)value;
        throw misuse("json obj is not string");
    }

    const std::vector<JSON>& JSON::asArray() const {
        if (isArray())
            return static_cast<const ArrayData*>(value)->data;
        throw misuse("json obj is not array");
    }

    const std::map<std::string, JSON>& JSON::asDictionary() const {
        if (isDictionary())
            return static_cast<const DictionaryData*>(value)->data;
        throw misuse("json obj is not dictionary");
    }

    Integer & JSON::asInteger() {
        if (isNull()) {
            value = new Integer();
            type = integer;
        }
        if (isInteger())
            return const_cast<Integer&>(const_cast<const JSON*>(this)->asInteger());
        throw misuse("JSON::asInteger() can't convert non-null to integer");
    }

    std::string &JSON::asString() {
        if (isNull()) {
            value = new std::string();
            type = string;
        }
        if (isString())
            return const_cast<std::string&>(const_cast<const JSON*>(this)->asString());
        throw misuse("JSON::asString() can't convert non-null to string");
    }

    std::vector<JSON> &JSON::asArray() {
        if (isNull()) {
            value = new ArrayData();
            type = array;
        }
        if (isArray())
            return const_cast<std::vector<JSON>&>(const_cast<const JSON*>(this)->asArray());
        throw misuse("JSON::asArray() can't convert non-null to array");
    }

    std::map<std::string, JSON> &JSON::asDictionary() {
        if (isNull()) {
            value = new DictionaryData();
            type = dictionary;
        }
        if (isDictionary())
            return const_cast<std::map<std::string, JSON>&>(const_cast<const JSON*>(this)->asDictionary());
        throw misuse("JSON::asDictionary() can't convert non-null to dictionary");
    }




    JSON& JSON::operator[](size_t index) {
        std::vector<JSON>& arr = asArray();
        if (arr.size() <= index)
            arr.resize(index + 1);
        return arr[index];
    }

    JSON& JSON::operator[](const std::string &key) {
        std::map<std::string, JSON>& dict = asDictionary();
        return dict[key];
    }

    const JSON& JSON::operator[](size_t index) const {
        if (!isArray() || asArray().size() <= index)
            throw misuse("Index out of range");
        return static_cast<ArrayData*>(value)->data[index];
    }

    const JSON& JSON::operator[](const std::string &key) const {
        if (!isDictionary() || asDictionary().count(key) != 1)
            throw misuse("No such key");
        return asDictionary().at(key);
    }

    bool JSON::operator==(const JSON &B) const {
        if (this == &B)
            return true;
        if (is_subtree_of(*this, B) || is_subtree_of(B, *this))
            return false;
        return eq_compare_json(*this, B);
    }

    bool JSON::operator!=(const JSON &B) const {
        return !eq_compare_json(*this, B);
    }
}
